#include "utils.h"

/**
 * Search the program configuration for the value of a specified field
 * @param key	key to query value for
 * @param buffer	character buffer to which to copy value found
 * @return	true if operation successful, false if not
 */
bool
get_config(std::string key, char buffer[])
{
	std::ifstream config;
	std::string value;
	bool key_found = false;

	config.open(CONFIG_NAME);
	if (!config.is_open())
		return false;

	while (config.good() && !key_found) {
		size_t delimiter;
		size_t key_start;
		std::string line;

		getline (config, line);
		if (!config.good())
			break;

		// Check if key exists in line
		key_start = line.find(key);
		if (key_start == std::string::npos)
			continue;

		// Check if delimiter exists in line
		delimiter = line.find("=");
		if (delimiter == std::string::npos)
			continue;

		// key must end one character before delimiter for this to be
		// a complete and certain match
		if (delimiter == key.length())
		{
			value = line.substr (delimiter + 1, std::string::npos);
			strcpy(buffer, value.c_str());
			key_found = true;
			break;
		}
	}
	config.close();
	return key_found;
}

// Set fb0/fb2 blend level
// 0 means no fb0; 255 means fully fb0
// Return true if operation was a success
// Note: call this AFTER gstreamer initializations: something in them resets the alpha
bool
set_alpha(int alpha_arg)
{
	int fb0_fd;
	int result;

	struct mxcfb_gbl_alpha alpha;
	alpha.enable = 1;
	alpha.alpha = alpha_arg;

	fb0_fd = open("/dev/fb0", O_RDWR, 0);
	if(fb0_fd < 0)
	{
		printf("\nCould not open /dev/fb0. Now exiting");
		exit(-1);
	}

	result = ioctl(fb0_fd, MXCFB_SET_GBL_ALPHA, &alpha);
	if(result < 0)
	{
		printf("\nUnable to set alpha for fb0. Now exiting");
		exit(-1);
	}

	close(fb0_fd);
	return true;
}

// Clears and wakes primary framebuffer (fb0)
// Returns true if successful
bool
wake()
{
	std::ofstream fb0_blank;
	fb0_blank.open("/sys/class/graphics/fb0/blank");
	fb0_blank << 1;
	fb0_blank.close();

	fb0_blank.open("/sys/class/graphics/fb0/blank");
	fb0_blank << 0;
	fb0_blank.close();

	if(fb0_blank.fail())
	{
		printf("\nFailed to open fb0 for clearing and waking. Now exiting");
		exit(-1);
	}

	std::ofstream fb2_blank;
	fb2_blank.open("/sys/class/graphics/fb2/blank");
	fb2_blank << 1;
	fb2_blank.close();

	fb2_blank.open("/sys/class/graphics/fb2/blank");
	fb2_blank << 0;
	fb2_blank.close();

	if(fb2_blank.fail())
	{
		printf("\nFailed to open fb2 for clearing and waking. Now exiting");
	}

	return true;
}

// Resizes fb2 overlay to the size of fb0
// Returns 'true' if successful
bool
resize_fb2()
{
	struct fb_var_screeninfo fb0_screeninfo;
	struct fb_var_screeninfo fb2_screeninfo;

	int fb0_fd;
	int fb2_fd;

	int fb2_new_xres;	// New resolutions for fb2 resolution
	int fb2_new_yres;	// Based on fb0 resolutions

	// Open fb0 to read screen dimensions
	printf("\nOpening /dev/fb0 for reading");
	fb0_fd = open("/dev/fb0", O_RDONLY);
	if(fb0_fd < 0)
	{
		printf("\nFailed to open /dev/fb0. Now exiting");
		exit(-1);
	}

	// Open fb2 to read and set screen dimensions
	printf("\nOpening /dev/fb2 for reading and writing");
	fb2_fd = open("/dev/fb2", O_RDWR);
	if(fb2_fd < 0)
	{
		printf("\nFailed to open /dev/fb2. Now exiting");
		exit(-1);
	}

	// Read fb0 screen dimensions
	printf("\nReading /dev/fb0 variable screen information");
	if(ioctl(fb0_fd, FBIOGET_VSCREENINFO, &fb0_screeninfo) < 0)
	{
		printf("\nFailed to obtain fb0 variable screen info. Now exiting");
		exit(-1);
	}

	printf("\nfb0 xres: %d", fb0_screeninfo.xres);
	printf("\nfb0 yres: %d", fb0_screeninfo.yres);

	// Set fb2 new resolutions
	fb2_new_xres = fb0_screeninfo.xres;
	fb2_new_yres = fb0_screeninfo.yres;

	printf("\nClosing fb0 file descriptor");
	close(fb0_fd);

	// Obtain fb2 variable screen info
	printf("\nObtaining /dev/fb2 variable screen information");
	if(ioctl(fb2_fd, FBIOGET_VSCREENINFO, &fb2_screeninfo) < 0)
	{
		printf("\nFailed to obtain fb2 variable screen info. Now exiting");
		exit(-1);
	}

	// Set fb2 variable screen info based on new fb2 resolutions
	printf("\nSetting fb2 resolutions to match fb0");
	fb2_screeninfo.xres = fb2_new_xres;
	fb2_screeninfo.yres = fb2_new_yres;

	// Write new resolution values to fb2
	printf("\nAttempting to \'put\' new values of xres and yres to fb2");
	if(ioctl(fb2_fd, FBIOPUT_VSCREENINFO, &fb2_screeninfo) < 0)
	{
		printf("\nError writing new resolution values to fb2. Now exiting");
		exit(-1);
	}

	// Close fb2 file descriptor, clean up
	printf("\nClosing fb2 file descriptor");
	printf("\n");
	close(fb2_fd);

	return true;
}
